local VERSION = tonumber(("$Revision: 31165 $"):match("%d+"))

local QuestsFu = QuestsFu
local QuestsFu_Blizz = QuestsFu:NewModule("Blizz", "AceEvent-2.0", "AceHook-2.1", "AceConsole-2.0")
if QuestsFu.revision < VERSION then
	QuestsFu.version = "r" .. VERSION
	QuestsFu.revision = VERSION
	QuestsFu.date = ("$Date: 2007-03-28 01:51:48 -0400 (Wed, 28 Mar 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local quixote = AceLibrary("Quixote-1.0")

local L = AceLibrary("AceLocale-2.2"):new("QuestsFu_Blizz")
L:RegisterTranslations("enUS", function() return {
	["Blizzard frames"] = true,
	["Show quest levels in the quest log and NPC dialogs"] = true,
} end)

function QuestsFu_Blizz:OnInitialize()
	self.db = QuestsFu:AcquireDBNamespace("Blizz")
	QuestsFu:RegisterDefaults("Blizz", "profile", {
		--["enabled"] = true,
		["complete"] = true,
	})
	QuestsFu.options.args.Blizz = {
		name = L["Blizzard frames"],
		desc = L["Show quest levels in the quest log and NPC dialogs"],
		type = "group",
		args = {
			toggle = {
				name = AceLibrary("AceLocale-2.2"):new("QuestsFu")["Enabled"],
				desc = AceLibrary("AceLocale-2.2"):new("QuestsFu")["Suspend/resume this module."],
				type = 'toggle',
				order = -1,
				get = function() return QuestsFu:IsModuleActive(self) end,
				set = function() QuestsFu:ToggleModuleActive(self) end
			},
		},
	}
end

function QuestsFu_Blizz:OnEnable()
	self:RegisterEvent("GOSSIP_SHOW", "OnGossipShow")
	self:RegisterEvent("QUEST_GREETING", "OnQuestGreeting")
	self:SecureHook("QuestLog_Update")
	--self:Hook("GetLFGTypeEntries", true)
end

----------------------------------------------------
-- Add quest levels to the quest log
----------------------------------------------------

function QuestsFu_Blizz:QuestLog_Update()
	--This is fragile, but lets us ignore event sequencing.  It'll break if anyone is meddling with the output of GetQuestLogTitle.
	--(questIndexes can change on every QUEST_LOG_UPDATE; this *will* be called before Quixote updates from that event, because Quixote is
	--throttling to avoid crashing the client on edge cases. I blame Blizzard for it being possible to crash the game by scanning the quest log. So.)
	for i = 1, QUESTS_DISPLAYED, 1 do
		--local questIndex = i + FauxScrollFrame_GetOffset(QuestLogListScrollFrame)
		local titleLine = _G["QuestLogTitle"..i]
		if titleLine and titleLine:GetText() then
			local title, level, tag = quixote:GetQuest(titleLine:GetText():trim())
			if title then
				titleLine:SetText(QuestsFu:MakeTag(level, tag)..title)
				local check = _G["QuestLogTitle" .. i .. "Check"]
				if check:IsVisible() then
					check:SetPoint("LEFT", titleLine, "LEFT", QuestLogDummyText:GetWidth() + 24, 0)
				end
			end
		end
	end
end

----------------------------------------------------
-- Add quest levels to LFG
----------------------------------------------------

--[[
-- Fuck you, Blizzard. ~vhaarr
function QuestsFu_Blizz:GetLFGTypeEntries(index)
	if index and tonumber(index) then
		index = tonumber(index)
		local original = {self.hooks["GetLFGTypeEntries"](index)}
		for i, v in ipairs(original) do
			if index == 2 or index == 3 or index == 5 then
				if (index == 5 and tourist:IsZone(v)) or tourist:IsInstance(v) then
					local low, high = tourist:GetLevel(v)
					if low and high then
						original[i] = v .. " ("..tostring(low).."-"..tostring(high)..")"
					end
				end
			elseif index == 4 then
				local _, level = quixote:GetQuestByName(v)
				if level and tonumber(level) then
					original[i] = "["..tostring(level).."] "..v
				end
			end
		end
		return unpack(original)
	end
	return self.hooks["GetLFGTypeEntries"](index)
end
]]

----------------------------------------------------
-- Add quest levels to NPC dialog when possible
----------------------------------------------------

--This is pretty much how Minimalist does it, which it credits to AutoSelect.
function QuestsFu_Blizz:OnGossipShow()
	local buttonindex = 1
	local list, button
	if GetGossipAvailableQuests() then
		list = {GetGossipAvailableQuests()}
		for i = 2,getn(list),2 do
			button = _G["GossipTitleButton"..buttonindex]
			button:SetText(format('[%d] %s',list[i],list[i-1]))
			buttonindex = buttonindex + 1
		end
		buttonindex = buttonindex + 1
		for k in pairs(list) do
			list[k] = nil
		end
	end
	if GetGossipActiveQuests() then
		list = {GetGossipActiveQuests()}
		for i = 2,getn(list),2 do
			button = _G["GossipTitleButton"..buttonindex]
			button:SetText(format('[%d] %s',list[i],list[i-1]))
			buttonindex = buttonindex + 1
		end
		for k in pairs(list) do
			list[k] = nil
		end
	end
end

--This is pretty much how Minimalist does it, which it credits to AutoSelect.
function QuestsFu_Blizz:OnQuestGreeting()
	local numactive,numavailable = GetNumActiveQuests(), GetNumAvailableQuests()
	local title, level, button
	local o,GetTitle,GetLevel = 0,GetActiveTitle,GetActiveLevel
	for i=1, numactive + numavailable do
		if i == numactive + 1 then
			o,GetTitle,GetLevel = numactive,GetAvailableTitle,GetAvailableLevel
		end
		title,level = GetTitle(i-o), GetLevel(i-o)
		button = _G["QuestTitleButton"..i]
		button:SetText(format('[%d] %s',level,title))
	end
end
